" Convert Vim help files to browsable HTML
" Full Vim script solution ! 
" Depends on Vim 2html.vim script.
" Author: David Blanchet <davidDOTblanchetATfreeDOTfr>

" Usage:
"  1. Drop this script in the directory that contains help files
" then
"  2. Open this file with gvim then ":so %"
"  3. Be patient and hit enter from time to time
" or
"  2. "gvim -s vim2html.vim"
"
" WARNING: because of a possible bug, please use "gvim" instead of "vim".

" Note to translators: {{{1
"  This script was created to simplify Vim manual conversion to HTML for
"  french translators of Vim manuals. Most of it is written in english.
"  Though, french language may be present in a few strings, when they are
"  meant to be seen by users.
"
"  Translation of the script is normally limited to a few places, which are:
"   - CreateNotFoundFile() function,
"   - AddOriginalFileLink() function,
"   - CreateHTMLFileList() function,
"   - CreatePHPHomeFile() function.
"  Unfortunately, text is mostly inside <html> tags. Please keep lines with
"  single HTML tags unchanged, so that the plugin can still perform needed
"  transformations to HTML.
"  A specific text tag has been added through code : "XXXTrans". It explains
"  what should be translated. You can use the "/XXXTrans" command to find them
"  out.
"  A very limited knowledge of PHP, HTML and Vim script language (just like
"  me!) should make translation more comfortable.      -- David

" HISTORY: {{{1
" 2004/05/03:	Creation.
" 2004/05/13:	Links with '<', '>' and '&' are resolved.
"		FTP links converted.
" 2004/05/16:	CSS list addition.
"		Variables and register cleanups.
"		Silent substitutions.
" 2004/05/21:	Improved CSS stuff : use CSS file in dir, choose CSS default.
"		Improved "notfound.html" file.
"		Escaped special chars in tag search.
"	        Fixed link search pattern (non-greedy).
"	        Added Validator link.
" 2004/06/14:	Fixed page title.
"		Added many TODO entries :)
" 2004/06/16:	Improved page title.
"		Added file list generation as home page.
"		Added note to translators.
" 2004/06/17:	Added option to add links to non converted file.
"		Added "index.php" home page generation.
" 2005/01/20:	Tuned a few option to speedup conversion.		
" 2005/01/28:	Added input and output file location.

" TODO: {{{1
"  - BUG: E670 with french version of files (cf. Dindinx mail)
"  - Improve CSS support (detection, default, etc.)
"    Provide a script to convert Vim colorscheme to CSS 
"  - Fix curious results with console Vim
"  - Probably many bugs to solve
"  - Uuencode special chars in links?
"  - Simplify script translation
"  - Improve index.php (regexp matching, multiple choices...)

" Problems (bugs?) with 2html.vim:
"  - with vim (but not with gvim), DTD link appears at begining of HTML page.
"  - HTML links delimiters needs improving? e.g.: usr_45.txt (/vim.sf.net)
"  - ftp:// and https:// are not converted to HTML links.
" }}}1 

" -------------------------------------
"  CONFIGURATION
" ------------------------------------- 

" Specify here the title each HTML file should
" receive. The title always contains the file
" name. Strings should be HTML formatted.
" Example : 
"   let s:HTML_title_pre = "Fichier &lt;"
"   let s:HTML_title_post = "&gt; - Manuel de Vim"
" produces : "Fichier <help.txt> - Manuel de Vim"
" ---
let s:HTML_title_pre = ""
let s:HTML_title_post = " - Manuel de Vim"

" Comment out to a link to original (non HTML)
" manual file at the top or/and the bottom of
" each converted files.
" Possible values : "top", "bottom", "both"
" Any other value is ignored.
" ---
"let s:HTML_add_link_to_txt_file = "both"

" Comment out to choose a specific default CSS.
" Default is "default.css".
" 2html.vim CSS is used if no CSS is found in the directory.
" ---
let s:HTML_default_css = "morning.css"

" Comment out to generate a list of available
" HTML help files called :
"  - "index.html" if following option (see below) is not enabled
"  - "filelist.html" otherwise.
" ---
"let s:HTML_create_file_list = 1

" Comment out to generate a PHP index file that
" provides:
"  - a basic PHP tag search engine,
"  - a link to available help files list (see above),
"  - a link to "help.txt.html".
" When this facility is enabled, the file list
" generation (s:HTML_create_file_list) is
" automatically activated.
" ---
let s:HTML_create_php_home_file = 1

" Comment out to add W3C validator link
" at the bottom of each converted page.
" ---
"let s:HTML_add_W3C_validator_link = 1

" File location
" Default is current directory.
" Target directory must exist.
" The final '/' is needed.
" ---
let s:HTML_input_files_dir = "../../../doc/fr62017/"
let s:HTML_output_files_dir = "html/"

" -------------------------------------
"  END OF CONFIGURATION
" ------------------------------------- 

" A few global variables {{{1
let s:listname = "__help_file_list__"
let s:tagfilename = s:HTML_input_files_dir . "tags"
let s:rega = @a
if !exists("s:HTML_input_files_dir")
  let s:HTML_input_files_dir = ""
endif
if !exists("s:HTML_output_files_dir")
  let s:HTML_output_files_dir = ""
endif

function! VimConvert2html() " {{{1
  source $VIMRUNTIME/syntax/2html.vim
endfunction

function! GotoBufWin (dest) " {{{1
" Move the cursor to the right buffer.
" a:dest : name of the buffer 
" Return : TRUE when succeeded.
  let destnr = bufwinnr(a:dest)
  if (destnr == -1)
    return 0
  endif
  exe destnr . 'wincmd w'
  return 1
endfunction

function! FindLinkTarget(ftag, fromfile) " {{{1
" Find the file the HTML link should lead to.
" a:ftag     : tag name
" a:fromfile : file that contains the tag 
" Return     : the file pointed by the tag
  call GotoBufWin(s:tagfilename) 
  " Vim help tags file format :
  " ^tagname\tfile\t\/\*target\*$
  let target = "^" . escape(a:ftag, '\~') . "\t.*$" 
  normal gg
  if (search(target, "W") == 0)
      " 404-like page for tags
      " that can't be found.
      call GotoBufWin(a:fromfile) 
      return "notfound"
  endif
  let result = substitute(getline('.'), '^.*\t\(\S*\)\t.*$', '\1', "")
  call GotoBufWin(a:fromfile) 
  return result
endfunction

function! AddLinks() " {{{1 
" Find convenient places to put HTML links and put them.
" Patterns are from "help.vim" file. 

  " Anchors.
  " Pattern modified according to 2html.vim conversion.
  silent %s/\*\([#-)!+-~]\+\)\*</<a name="\1">\*\1*<\/a></ge

  " Add hyperlinks for FTP URL.
  " 2html.vim does it for HTTP links only.
  " NOTE: in usr_45.txt, 2html.vim give a bad
  " conversion because of final parenthesis.
  silent %s+\(ftp://\S\{-}\)\(\([.,;:]\=\(\s\|$\)\)\|[\\"'<>]\|&gt;\|&lt;\)+<a href="\1">\1</a>\2+ge
  
  " Options and Links.
  silent %s/<span[^>]*>\('[a-z]\{2,\}'\)<\/span>/<a href="#\1">&<\/a>/ge
  silent %s/<span[^>]*>\\\@<!|\([#-)!+-~]\+\)|<\/span>/<a href="#\1">&<\/a>/ge

  " Update links target according to help tag file.
  normal gg
  let thisfile = expand("%")
  while (search('href="#', 'W') > 0)
    let htag = substitute(getline('.'), '.\{-}="#\([^"]*\)".*', '\1', '')
    " Tag file keeps '<', '>' and '&' chars unchanged.
    let htag = substitute(htag, '&gt;', '>', '')
    let htag = substitute(htag, '&lt;', '<', '')
    let htag = substitute(htag, '&amp;', '&', '')
    let targetfile = FindLinkTarget(htag, thisfile) . ".html"
    silent .,.s/href="#/\='href="' . targetfile . '#'/e
  endwhile

endfunction

function! GetCSSList() " {{{1
" Build a HTML formatted list of CSS files.
  let lst = substitute(globpath(".", "*.css"), '\..\(\f*\).css',
	\'<link title="\1" rel="alternate stylesheet" type="text/css"' .
	\' media="screen" href="\1.css">', 'g')

  " Provide a default CSS.
  if !exists("s:HTML_default_css") 
    " Default :
    let lst = substitute(lst, 'title="default" rel="alternate ',
	  \'title="default" rel="', '')
  else
    " User defined :
    let name = substitute(s:HTML_default_css, '\(\).css', '\1', '') 
    let lst = substitute(lst, 'title="' . name . '" rel="alternate ',
	  \'title="' . name . '" rel="', '')
  endif

  " Take care of a possible printing CSS.
  if (stridx(lst, "print.css") != -1)
    let lst = substitute(lst, 'title="print" rel="alternate ', 'rel="', 'g')
    let lst = substitute(lst, 'media="screen" href="print.css"',
	  \'media="print" href="print.css"', 'g')
  endif

  return lst
endfunction

function! AddCSSList() " {{{1
" Add the HTML CSS list to the current buffer.
  if !exists("s:csslist")
    let s:csslist = GetCSSList()
  endif
  if s:csslist == ""
    " Keep 2html.vim style if no CSS is found.
    return
  endif
  " Suppress current style.
  0
  /<\/style/
  mark A
  ?<style?
  silent .,'Adelete
  " Add our own.
  let @a = s:csslist
  silent put! a
endfunction

"function! AddValidatorLink() {{{1
if exists("s:HTML_add_W3C_validator_link")
" Add W3C HTML 4.01 validator link at the end of current buffer.
  function! AddValidatorLink()
    normal G?</body?
    normal kO<p><a href="http://validator.w3.org/check?uri=referer">
	  \<img src="http://www.w3.org/Icons/valid-html401"
	  \ alt="Valid HTML 4.01!" height="31" width="88"></a></p>
  endfunction
endif

function! UpdateTitle() " {{{1
" Replace page title with the chosen title.
  0
  silent %s+<title>.*[/\\]\(\f\+\.txt\)\.html</title>+\="<title>"
	\ . s:HTML_title_pre . submatch(1) . s:HTML_title_post . "</title>"+e
endfunction

"function! AddOriginalFileLink() {{{1
if exists("s:HTML_add_link_to_txt_file")
" Insert a link to non converted file,
" accordingly to s:HTML_add_link_to_txt_file.
  function! AddOriginalFileLink()
    " XXXTrans : "Click to get plain-text version" link.
    let @a = '<p><a href="' . substitute(expand("%"),
	  \'.*[/\\]\(\f\+\.txt\)\.html', '\1', '') .
	  \'">(cliquez ici pour la version texte seul)</a></p>'
    if (s:HTML_add_link_to_txt_file == "bottom"
	  \|| s:HTML_add_link_to_txt_file == "both" )
      $
      ?</body?
      put! a
    endif
    if (s:HTML_add_link_to_txt_file == "top" 
	  \|| s:HTML_add_link_to_txt_file == "both" )
      0
      /<body/
      put a
    endif
  endfunction
endif
    
function! OpenTagFile() " {{{1
" Open tag file containing tags from help files in the directory.
  if !filereadable(s:tagfilename)
    exe "helptag " . s:HTML_input_files_dir
  endif
  exe "split " . s:tagfilename
  set noswapfile
  set switchbuf=useopen
  normal ggO
endfunction

function! CreateHelpFileList(listname) " {{{1
" Open a buffer with the list of help file.
" a:listname : name of the corresponding buffer 
  execute "split " . a:listname
  setlocal buftype=nofile
  setlocal noswapfile
  setlocal switchbuf=useopen
  let @a = globpath(s:HTML_input_files_dir . ".", "*.txt")
  put a
  %g/^$/d
  %s/^.*[/\\]\(\w*\.txt\)/\1/g
  nohls
endfunction

function! CreateNotFoundFile() "{{{1
" Create a 404-Not Found file.
  if filereadable("notfound.html")
    return
  endif
  exe "split " . s:HTML_output_files_dir . "notfound.html"
  " XXXTrans : page title when a file is not found.
  normal a<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	\ "http://www.w3.org/TR/html4/strict.dtd"><html><head><title>
	\Manuels de Vim</title>
  normal o<style>
  normal o</style>
  normal o</head><body>
  " XXXTrans : page content when a file is not found.
  normal o<h1>Page non trouv&eacute;e</h1>D&eacute;sol&eacute;,
	\ cette section du manuel de Vim est introuvable.
  normal o<p>Causes possibles :<ul><li>Le fichier n'est pas encore traduit,
	\</li><li>La conversion HTML a rencontr&eacute;
	\ un problme technique.</li></ul></p> 
  " XXXTrans : end of page content translation.
  normal o</body></html>
  call AddCSSList()
  silent write
  bwipeout %
endfunction

"function! CreateHTMLFileList() {{{1
if exists("s:HTML_create_file_list") || exists("s:HTML_create_php_home_file")
" Create a HTML file that contains the
" available converted files.
" a:filename : name of the HTML file 
  function! CreateHTMLFileList(filename)
    let @a = globpath(s:HTML_output_files_dir . '.', '*.txt.html')
    exe "split " . s:HTML_output_files_dir . a:filename
    put a
    silent %s/\..\(\f*\)$/\1/ge
    silent %g/^$/d
    silent %s/^\(\f*\)\.html$/<a href="\1.html">\1<\/a>/ge
    " XXXTrans : Available file list page title.
    normal ggO<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	  \ "http://www.w3.org/TR/html4/strict.dtd"><html><head>
	  \<title>Manuels de Vim - Liste des fichiers disponibles</title>
    normal o<style>
    normal o</style>
    normal o</head><body>
    " XXXTrans : Available file list main title.
    normal o<h1>Liste des fichiers disponibles :</h1><pre>
    normal Go</pre><hr/></body></html>
    call AddCSSList()
    silent write
    bwipeout! %
  endfunction
endif

"function! CreatePHPHomeFile() {{{1
if exists("s:HTML_create_php_home_file")
" Create a PHP file that offer limited
" a tag search engine.
  function! CreatePHPHomeFile()
    split index.php
    normal ggO<?php
    normal oif (isset($tag) and strlen($tag) > 0) {
    normal o$tags = file_get_contents("tags");
    normal o$pattern = "/\n" . addslashes(urldecode($tag)) . "\\t(.+)\\t\/\*/";
    normal oif (preg_match($pattern, $tags, $line)) {
    normal o$URL = $line[1] . ".html#" . $tag;
    normal oheader ("Location: $URL");
    normal o} else {
    normal ounset($line); // Needed for further computing, see below.
    normal o}
    normal o}
    normal o?>
    normal o<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
	  \ "http://www.w3.org/TR/html4/strict.dtd">
    " XXXTrans : Home page title.
    normal o<html><head><title>Manuels de Vim - Accueil</title>
    normal o<style>
    normal o</style>
    " XXXTrans : Home page content. Default provides :
    "  - a link to "help.txt.html" as main help file,
    "  - a link to "filelist.html" as available file list,
    "  - a textfield to search for a specific tag.
    normal o</head> <body> <h1>Acc&egrave;s aux manuels Vim traduits</h1>
	  \ <p>Cette page vous donne trois possibilit&eacute;s :
	  \ <ul> <li>Pour acc&eacute;der directement au fichier principal
	  \ de l'aide, suivez ce <a href="help.txt.html">lien</a>.</li>
	  \ <li>Pour acc&eacute;der &agrave; liste de fichiers d'aide
	  \ disponibles, suivez plut&ocirc;t ce <a href="filelist.html">
	  \lien</a>.</li> <li>Sinon, vous pouvez saisir un sujet pour y
	  \ acc&eacute;der plus rapidement.
    " XXXTrans : End of page content translation.
    normal o<?php
    normal oif (isset($tag) && !isset($line)) {
    normal oecho "<p><b>Le sujet '", $tag,
	  \ "' n'a pas &eacute;t&eacute; trouv&eacute;.</b></p>";
    normal o}
    normal o?>
    " XXXTrans : Submit button, translate the "value" field of <input
    " type="submit" ...> HTML tag.
    normal o<form method="GET" action="./index.php"> <input type="text"
	  \ value="<?php if (isset($tag)) {echo $tag;} ?>" name="tag">
	  \ <input type="submit" value="Rechercher"> </form> </li>
	  \ </ul> <hr/> </body> </html>
    call AddCSSList()
    exe "silent write " . s:HTML_output_files_dir . " %"
    bwipeout! %
  endfunction
endif
" }}}

" Initialization {{{1
call OpenTagFile()
call CreateHelpFileList(s:listname)

" Any option or command to speedup
" conversion may be added here :
set lazyredraw
set noswapfile
set undolevels=-1
set nohidden

" Help files conversion to HTML {{{1
let html_use_css = 1
exe "normal Go\<Esc>gg"
let s:line = 1
while (getline(s:line) != getline("$"))
  let s:name = getline(s:line)
  exe "split " . s:HTML_input_files_dir . s:name
  " ---------------------------------
  " Help file computing starts here.
  call VimConvert2html()
  call AddLinks()
  call AddCSSList()
  call UpdateTitle()
  if exists("s:HTML_add_link_to_txt_file")
    call AddOriginalFileLink()
  endif
  if exists("s:HTML_add_W3C_validator_link")
    call AddValidatorLink()
  endif
  " Add your own changes above this comment.
  " ---------------------------------
  execute "write " . s:HTML_output_files_dir . expand("%:t")

  bwipeout %
  exe "bwipeout " . s:name
  exe "sbuffer " . s:listname
  let s:line = s:line + 1
endwhile

" Additionnal files generation {{{1
call CreateNotFoundFile()
if exists("s:HTML_create_php_home_file")
  call CreateHTMLFileList("filelist.html")
  call CreatePHPHomeFile()
else
  if exists("s:HTML_create_file_list")
    call CreateHTMLFileList("index.html")
  endif
endif

" Final cleanups {{{1
exe "bwipeout! " . s:tagfilename
exe "bwipeout! " . s:listname
call delete(s:tagfilename)

let @a = s:rega
unlet s:rega s:name s:listname s:tagfilename s:line
unlet s:HTML_input_files_dir s:HTML_output_files_dir
unlet! s:HTML_add_W3C_validator_link s:HTML_default_css
      \ s:HTML_create_file_list s:HTML_create_php_home_file
      \ s:HTML_title_post s:HTML_title_pre s:HTML_add_link_to_txt_file
" }}}

"quit

" vim:sw=2:fdm=marker:
